@@ -15,18 +15,14 @@ Like all other perl parts, based on the pre-existing python generators
 
 @copyright: 2002-2004 D.H. aka crazyinsomniac on sourceforge.net
 @copyright: 2012-2016 Carsten Grohmann
+@copyright: 2017 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
-import os
-import os.path
-import re
-import types
-
-from codegen import BaseLangCodeWriter, \
-                    BaseSourceFileContent, \
-                    BaseWidgetHandler
+import os, os.path, re
+from codegen import BaseLangCodeWriter, BaseSourceFileContent, BaseWidgetHandler
 import wcodegen
+import compat
 
 
 class SourceFileContent(BaseSourceFileContent):
@@ -104,12 +100,10 @@ class SourceFileContent(BaseSourceFileContent):
 
             result = self.rec_class_decl.match(line)
             if result:
-##                self._logger.debug(">> class %r", result.group(1))
                 if not self.class_name:
                     # this is the first class declared in the file: insert the
                     # new ones before this
-                    out_lines.append('<%swxGlade insert new_classes>' %
-                                     self.nonce)
+                    out_lines.append( '<%swxGlade insert new_classes>' % self.nonce )
                     self.new_classes_inserted = True
                 self.class_name = result.group(1)
                 self.class_name = self.format_classname(self.class_name)
@@ -119,13 +113,7 @@ class SourceFileContent(BaseSourceFileContent):
             elif not inside_block:
                 result = self.rec_block_start.match(line)
                 if result:
-##                     self.logger.debug(">> block %r %r %r",
-##                         result.group('spaces'),
-##                         result.group('classname'),
-##                         result.group('block'),
-##                         )
-                    # replace the lines inside a wxGlade block with a tag that
-                    # will be used later by add_class
+                    # replace the lines inside a wxGlade block with a tag that will be used later by add_class
                     spaces = result.group('spaces')
                     which_class = result.group('classname')
                     which_block = result.group('block')
@@ -136,11 +124,9 @@ class SourceFileContent(BaseSourceFileContent):
                     self.spaces[which_class] = spaces
                     inside_block = True
                     if not self.class_name:
-                        out_lines.append('<%swxGlade replace %s>' % \
-                                         (self.nonce, which_block))
+                        out_lines.append( '<%swxGlade replace %s>' % (self.nonce, which_block) )
                     else:
-                        out_lines.append('<%swxGlade replace %s %s>' % \
-                                         (self.nonce, which_class, which_block))
+                        out_lines.append( '<%swxGlade replace %s %s>' % (self.nonce, which_class, which_block) )
                 else:
                     result = self.rec_event_handler.match(line)
                     if result:
@@ -150,53 +136,36 @@ class SourceFileContent(BaseSourceFileContent):
                             which_class, {})[which_handler] = 1
                     if self.class_name and self.is_end_of_class(line):
                         # add extra event handlers here...
-                        out_lines.append('<%swxGlade event_handlers %s>'
-                                         % (self.nonce, self.class_name))
+                        out_lines.append( '<%swxGlade event_handlers %s>' % (self.nonce, self.class_name) )
                     out_lines.append(line)
                     if self.is_import_line(line):
                         # add a tag to allow extra modules
-                        out_lines.append('<%swxGlade extra_modules>\n'
-                                         % self.nonce)
+                        out_lines.append( '<%swxGlade extra_modules>\n' % self.nonce )
             else:
                 # ignore all the lines inside a wxGlade block
                 if self.rec_block_end.match(line):
 ##                     self._logger.debug('end block')
                     inside_block = False
         if not self.new_classes_inserted:
-            # if we are here, the previous ``version'' of the file did not
-            # contain any class, so we must add the new_classes tag at the
-            # end of the file
-            out_lines.append('<%swxGlade insert new_classes>' % self.nonce)
+            # if we are here, the previous ``version'' of the file did not contain any class, so we must add the
+            # new_classes tag at the end of the file
+            out_lines.append( '<%swxGlade insert new_classes>' % self.nonce )
         # set the ``persistent'' content of the file
-        self.content = "".join(out_lines)
+        self.content = out_lines
 
     def is_import_line(self, line):
         return line.lstrip().startswith('use Wx')
 
-# end of class SourceFileContent
 
 
 class WidgetHandler(BaseWidgetHandler):
-    """\
-    Interface the various code generators for the widgets must implement
-    """
+    "Interface the various code generators for the widgets must implement"
+    new_signature = []  # Constructor signature ($self->SUPER::new(@stuff)); see PerlCodeWriter.new_defaults
 
-    new_signature = []
-    """\
-    Constructor signature ($self->SUPER::new(@stuff))
-
-    @see: L{PerlCodeWriter.new_defaults}
-    """
-
-# end of class WidgetHandler
 
 
 class PerlCodeWriter(BaseLangCodeWriter, wcodegen.PerlMixin):
-    """\
-    Code writer class for writing Perl code out of the designed GUI elements
-
-    @see: L{BaseLangCodeWriter}
-    """
+    "Code writer class for writing Perl code out of the designed GUI elements; see: BaseLangCodeWriter"
 
     _code_statements = {
         'backgroundcolour': "%(objname)s->SetBackgroundColour(%(value)s);\n",
@@ -210,18 +179,13 @@ class PerlCodeWriter(BaseLangCodeWriter, wcodegen.PerlMixin):
         'tooltip':          "%(objname)s->SetToolTipString(%(tooltip)s);\n",
         'tooltip_3':        "%(objname)s->SetToolTip(%(tooltip)s);\n",
         'wxcolour':         "Wx::Colour->new(%(value)s)",
+        'wxnullcolour':     "Wx::NullColour",
         'wxsystemcolour':   "Wx::SystemSettings::GetColour(%(value)s)",
         }
 
     class_separator = '::'
-    classattr_always = ['wxBoxSizer', 'wxStaticBoxSizer', 'wxGridSizer',
-                        'wxFlexGridSizer']
+    classattr_always = ['wxBoxSizer', 'wxStaticBoxSizer', 'wxGridSizer', 'wxFlexGridSizer']
 
-    global_property_writers = {
-        'font':            BaseLangCodeWriter.FontPropertyHandler,
-        'events':          BaseLangCodeWriter.EventsPropertyHandler,
-        'extraproperties': BaseLangCodeWriter.ExtraPropertiesPropertyHandler,
-        }
 
     indent_amount = 1
     indent_symbol = '\t'
@@ -232,10 +196,7 @@ class PerlCodeWriter(BaseLangCodeWriter, wcodegen.PerlMixin):
 
     name_ctor = 'new'
 
-    new_defaults = []
-    """\
-    Default class members, will be initialised during L{initialize()}
-    """
+    new_defaults = []  # Default class members, will be initialised during new_project()
 
     shebang = '#!/usr/bin/perl -w -- \n#\n'
 
@@ -249,11 +210,8 @@ class PerlCodeWriter(BaseLangCodeWriter, wcodegen.PerlMixin):
                      '}\n' \
                      '\n'
 
-    tmpl_class_end = '\n' \
-                     '%(comment)s end of class %(klass)s\n' \
-                     '\n' \
-                     '1;\n' \
-                     '\n'
+    tmpl_class_end = '\n%(comment)s end of class %(klass)s\n\n1;\n\n'
+    tmpl_class_end_nomarker = '\n\n1;\n\n'
 
     tmpl_ctor_call_layout = '\n' \
                             '%(tab)s$self->__set_properties();\n' \
@@ -286,111 +244,75 @@ sub %(handler)s {
     tmpl_func_empty = '%(tab)sreturn;\n'
 
     tmpl_sizeritem = '%s->Add(%s, %s, %s, %s);\n'
+    tmpl_gridbagsizeritem = '%s->Add(%s, %s, %s, %s, %s);\n'
+    tmpl_gridbagsizerspacer = '%s->Add(%s, %s, %s, %s, %s, %s);\n'
+    tmpl_spacersize = '%s, %s'
 
     tmpl_style = \
-        '%(tab)s$style = %(style)s \n' \
+        '%(tab)s$style = %(style)s\n' \
         '%(tab)s%(tab)sunless defined $style;\n' \
         '\n'
+    tmpl_toplevel_style = tmpl_style
 
     tmpl_appfile = """%(overwrite)s%(header_lines)s"""
 
-    tmpl_detailed = """\
-package %(klass)s;
+    def _get_app_template(self, app, top_win):
+        'build template string for application'
+        if not self.app_name: return None
 
-use base qw(Wx::App);
-use strict;
-%(pl_import)s
-sub OnInit {
-%(tab)smy( $self ) = shift;
+        # XXX use Show() for frames/panels and ShowModal()/Destroy for dialogs
+        klass = app.klass
+        
+        if self._use_gettext:
+            gettext1 = ['%(tab)smy $local = Wx::Locale->new("English", "en", "en"); # replace with ??',
+                        '%(tab)s$local->AddCatalog("%(textdomain)s"); # replace with the appropriate catalog name\n']
+        else:
+            gettext1 = []
 
-%(tab)sWx::InitAllImageHandlers();
-
-%(tab)smy $%(top_win)s = %(top_win_class)s->new();
-
-%(tab)s$self->SetTopWindow($%(top_win)s);
-%(tab)s$%(top_win)s->Show(1);
-
-%(tab)sreturn 1;
-}
-# end of class %(klass)s
-
-package main;
-
-unless(caller){
-%(tab)smy $%(name)s = %(klass)s->new();
-%(tab)s$%(name)s->MainLoop();
-}
-"""
-
-    tmpl_gettext_detailed = """\
-package %(klass)s;
-
-use base qw(Wx::App);
-use strict;
-%(pl_import)s
-sub OnInit {
-%(tab)smy( $self ) = shift;
-
-%(tab)sWx::InitAllImageHandlers();
-
-%(tab)smy $%(top_win)s = %(top_win_class)s->new();
-
-%(tab)s$self->SetTopWindow($%(top_win)s);
-%(tab)s$%(top_win)s->Show(1);
-
-%(tab)sreturn 1;
-}
-# end of class %(klass)s
-
-package main;
-
-unless(caller){
-%(tab)smy $local = Wx::Locale->new("English", "en", "en"); # replace with ??
-%(tab)s$local->AddCatalog("%(textdomain)s"); # replace with the appropriate catalog name
-
-%(tab)smy $%(name)s = %(klass)s->new();
-%(tab)s$%(name)s->MainLoop();
-}
-"""
-
-    tmpl_simple = """\
-1;
-
-package main;
-%(pl_import)s
-unless(caller){
-%(tab)slocal *Wx::App::OnInit = sub{1};
-%(tab)smy $%(name)s = Wx::App->new();
-%(tab)sWx::InitAllImageHandlers();
-
-%(tab)smy $%(top_win)s = %(top_win_class)s->new();
-
-%(tab)s$%(name)s->SetTopWindow($%(top_win)s);
-%(tab)s$%(top_win)s->Show(1);
-%(tab)s$%(name)s->MainLoop();
-}
-"""
-
-    tmpl_gettext_simple = """\
-1;
-
-package main;
-%(pl_import)s
-unless(caller){
-%(tab)smy $local = Wx::Locale->new("English", "en", "en"); # replace with ??
-%(tab)s$local->AddCatalog("%(textdomain)s"); # replace with the appropriate catalog name
-
-%(tab)slocal *Wx::App::OnInit = sub{1};
-%(tab)smy $%(name)s = Wx::App->new();
-%(tab)sWx::InitAllImageHandlers();
-
-%(tab)smy $%(top_win)s = %(top_win_class)s->new();
-
-%(tab)s$%(name)s->SetTopWindow($%(top_win)s);
-%(tab)s$%(top_win)s->Show(1);
-%(tab)s$%(name)s->MainLoop();
-}
-"""
+        if klass:
+            ret = [ 'package %(klass)s;',
+                    '',
+                    'use base qw(Wx::App);',
+                    'use strict;',
+                    '%(pl_import)s',
+                    'sub OnInit {',
+                    '%(tab)smy( $self ) = shift;',
+                    '',
+                    '%(tab)sWx::InitAllImageHandlers();',
+                    '',
+                    '%(tab)smy $%(top_win)s = %(top_win_class)s->new();',
+                    '',
+                    '%(tab)s$self->SetTopWindow($%(top_win)s);',
+                    '%(tab)s$%(top_win)s->Show(1);',
+                    '',
+                    '%(tab)sreturn 1;',
+                    '}']
+            if self._mark_blocks:
+                ret.append('# end of class %(klass)s')
+            ret += ['',
+                    'package main;',
+                    '',
+                    'unless(caller){'] + gettext1 + [
+                    '%(tab)smy $%(name)s = %(klass)s->new();',
+                    '%(tab)s$%(name)s->MainLoop();',
+                    '}', '']
+        else:
+            ret = ['1;',
+                    '',
+                    'package main;',
+                    '%(pl_import)s',
+                    'unless(caller){'] + gettext1 + [
+                    '%(tab)slocal *Wx::App::OnInit = sub{1};',
+                    '%(tab)smy $%(name)s = Wx::App->new();',
+                    '%(tab)sWx::InitAllImageHandlers();',
+                    '',
+                    '%(tab)smy $%(top_win)s = %(top_win_class)s->new();',
+                    '',
+                    '%(tab)s$%(name)s->SetTopWindow($%(top_win)s);',
+                    '%(tab)s$%(top_win)s->Show(1);',
+                    '%(tab)s$%(name)s->MainLoop();',
+                    '}', '']
+        return '\n'.join(ret)
 
     def init_lang(self, app_attrs):
         # initial new defaults late to use the proper indent characters
@@ -406,17 +328,17 @@ unless(caller){
             }
 
         self.header_lines = [
-            'use Wx 0.15 qw[:allclasses];\n',
+            'use Wx qw[:allclasses];\n',
             'use strict;\n'
         ]
 
-    def add_app(self, app_attrs, top_win_class):
+    def add_app(self, app_attrs, top_win):
         # add language specific mappings
         if self.multiple_files:
-            self.lang_mapping['pl_import'] = "\nuse %s;\n" % top_win_class
+            self.lang_mapping['pl_import'] = "\nuse %s;\n" % top_win.klass
         else:
             self.lang_mapping['pl_import'] = ''
-        BaseLangCodeWriter.add_app(self, app_attrs, top_win_class)
+        BaseLangCodeWriter.add_app(self, app_attrs, top_win)
 
     def generate_code_ctor(self, code_obj, is_new, tab):
         code_lines = []
@@ -427,9 +349,8 @@ unless(caller){
         mycn_f = getattr(builder, 'cn_f', self.cn_f)
 
         # custom base classes support
-        custom_base = getattr(code_obj, 'custom_base',
-                              code_obj.properties.get('custom_base', None))
-        if code_obj.preview or (custom_base and not custom_base.strip()):
+        custom_base = getattr( code_obj, 'custom_base', code_obj.properties.get('custom_base', None) )
+        if self.preview or (custom_base and not custom_base.strip()):
             custom_base = None
 
         new_signature = getattr(builder, 'new_signature', [])
@@ -437,24 +358,20 @@ unless(caller){
         # generate constructor code
         if is_new:
             write('package %s;\n\n' % code_obj.klass)
-            write('use Wx qw[:everything];\nuse base qw(%s);\nuse strict;\n\n'
-                    % code_obj.base.replace('wx', 'Wx::', 1))
+            write('use Wx qw[:everything];\nuse base qw(%s);\nuse strict;\n\n' % code_obj.base.replace('wx', 'Wx::', 1))
 
             if self._use_gettext:
                 if self.multiple_files:
-                    self.classes[code_obj.klass].dependencies[
-                        "use Wx::Locale gettext => '_T';\n"] = 1
+                    self.classes[code_obj.klass].dependencies["use Wx::Locale gettext => '_T';\n"] = 1
                 else:
                     write("use Wx::Locale gettext => '_T';\n")
 
-            # The dependencies have to add to the package block too
-            # because global imports are not visible inside the package
-            # block
+            # The dependencies have to add to the package block too because global imports are not visible inside the
+            # package block
             # TODO: Don't add dependencies twice with Perl
 
             # write the module dependencies for this class (package)
-            dep_list = self.classes[code_obj.klass].dependencies.keys()
-            dep_list.sort()
+            dep_list = sorted( self.classes[code_obj.klass].dependencies.keys() )
             if dep_list:
                 code = self._tagcontent('dependencies', dep_list, True)
                 write(code)
@@ -465,44 +382,39 @@ unless(caller){
 
             if new_signature:
                 for k in new_signature:
-                    if self.new_defaults.has_key(k):
+                    if k in self.new_defaults:
                         write(self.new_defaults[k])
             else:
                 new_signature = ['@_[1 .. $#_]']  # shift(@_)->SUPER::new(@_);
-                self._logger.info(
-                    "%s did not declare self.new_defaults ", code_obj.klass
-                    )
+                self._logger.info( "%s did not declare self.new_defaults ", code_obj.klass )
 
         elif custom_base:
             # custom base classes set, but "overwrite existing sources" not
             # set. Issue a warning about this
-            self.warning(
-                '%s has custom base classes, but you are not overwriting '
-                'existing sources: please check that the resulting code is '
-                'correct!' % code_obj.name
-                )
+            self.warning( '%s has custom base classes, but you are not overwriting existing sources: '
+                          'please check that the resulting code is correct!' % code_obj.name )
 
-        # __init__ begin tag
-        write(self.tmpl_block_begin % {
-            'class_separator': self.class_separator,
-            'comment_sign':    self.comment_sign,
-            'function':        self.name_ctor,
-            'klass':           self.cn_class(code_obj.klass),
-            'tab':             tab,
-            })
+        if self._mark_blocks:
+            # __init__ begin tag
+            write(self.tmpl_block_begin % {'class_separator':self.class_separator, 'comment_sign':self.comment_sign,
+                                           'function':self.name_ctor, 'klass':self.cn_class(code_obj.klass),
+                                           'tab':tab} )
 
-        prop = code_obj.properties
-        style = prop.get("style", None)
-        if style:
-            stmt_style = self._format_style(style, code_obj)
-            write(stmt_style % {
-                'style': mycn_f(style),
-                'tab': tab,
-                })
+        style_p = code_obj.properties.get("style")
+        if style_p and style_p.value_set != style_p.default_value:
+            style = style_p.get_string_value()
+            m_style = mycn_f( style )
+            if m_style:
+                stmt_style = self._format_style(style, code_obj)
+                write( stmt_style % {'style':m_style, 'tab':tab} )
 
         # class parent constructor
         write(tab + '$self = $self->SUPER::new( %s );\n' % ", ".join(new_signature))
 
+        # set size here to avoid problems with splitter windows
+        if 'size' in code_obj.properties and code_obj.properties["size"].is_active():
+            write( tab+ self.generate_code_size(code_obj) )
+
         # classes[code_obj.klass].deps now contains a mapping of child to
         # parent for all children we processed...
         object_order = []
@@ -542,16 +454,10 @@ unless(caller){
                 win_id = '$self->{%s}->GetId' % win_id[8:]
 
             if 'EVT_NAVIGATION_KEY' in event:
-                tmpl = '%(tab)s%(event)s($self, \\&%(handler)s);\n'
+                tmpl = '''%(tab)s%(event)s($self, $self->can('%(handler)s'));\n'''
             else:
-                tmpl = '%(tab)s%(event)s($self, %(win_id)s, ' \
-                       '\\&%(handler)s);\n'
-            details = {
-                'tab': tab,
-                'event': self.cn(event),
-                'handler': handler,
-                'win_id': win_id,
-                }
+                tmpl = '''%(tab)s%(event)s($self, %(win_id)s, $self->can('%(handler)s'));\n'''
+            details = { 'tab': tab, 'event': self.cn(event), 'handler': handler, 'win_id': win_id }
             write(tmpl % details)
 
         if event_handlers:
@@ -561,9 +467,12 @@ unless(caller){
 
     def generate_code_id(self, obj, id=None):
         if id is None:
-            id = obj.properties.get('id')
+            id = obj.window_id
         if not id:
+            if obj is not None and "stockitem" in obj.properties and obj.stockitem:
+                return '', self.cn("wxID_" + obj.stockitem)
             return '', self.cn('wxID_ANY')
+        id = str(id)
         tokens = id.split('=', 1)
         if len(tokens) == 2:
             name, val = tokens
@@ -582,7 +491,7 @@ unless(caller){
 
     def generate_code_size(self, obj):
         objname = self.format_generic_access(obj)
-        size = obj.properties.get('size', '').strip()
+        size = obj.properties["size"].get_string_value()
         use_dialog_units = (size[-1] == 'd')
         if not obj.parent:
             method = 'SetSize'
@@ -617,7 +526,7 @@ unless(caller){
         s = s.replace('@', r'\@')
 
         # convert all strings to unicode first
-        if not isinstance(s, types.UnicodeType):
+        if not isinstance(s, compat.unicode):
             s = s.decode(self.app_encoding)
 
         # check if it's pure ascii
@@ -631,10 +540,12 @@ unless(caller){
             pass
 
         # convert unicode strings to pure ascii
-        # use "raw-unicode-escape" just escaped unicode characters and not
-        # default escape sequences
+        # use "raw-unicode-escape" just escaped unicode characters and not default escape sequences
         s = s.encode('raw-unicode-escape')
         s = self._recode_x80_xff(s)
+        if compat.PYTHON3:
+            # convert back to str (unicode)
+            s = s.decode("ASCII")
         # convert Python style to Perl style
         s = re.sub(r'\\u([0-9a-f]{4})', r'\\N{U+\1}', s)
 
@@ -658,7 +569,7 @@ unless(caller){
         elif obj.klass == 'spacer':
             return obj.name
         # Perl stores sizers always in class attributes
-        elif self.test_attribute(obj) or obj.in_sizers:
+        elif self.store_as_attr(obj) or obj.is_sizer:
             return '$self->{%s}' % obj.name
         return '$%s' % obj.name
 
@@ -667,35 +578,16 @@ unless(caller){
         return stmt
 
     def _get_class_filename(self, klass):
-        """
-        Returns the name for a Perl module (.pm) to store a single class
-        in multi file projects.
-
-        @param klass: Class name
-        @type klass:  String
-        
-        @rtype: String
-        """
-        filename = os.path.join(
-            self.out_dir,
-            klass.replace('::', os.sep) + '.pm'
-            )
+        "Returns the name for a Perl module (.pm) to store a single class in multi file projects"
+        filename = os.path.join( self.out_dir, klass.replace('::', os.sep) + '.pm' )
         return filename
 
     def format_generic_access(self, obj):
         if obj.is_toplevel:
             return '$self'
-        else:
-            return self._format_classattr(obj)
+        return self._format_classattr(obj)
 
-# end of class PerlCodeWriter
 
-writer = PerlCodeWriter()
-"""\
-The code writer is an instance of L{PerlCodeWriter}.
-"""
+writer = PerlCodeWriter()  # the code writer instance
 
-language = writer.language
-"""\
-Language generated by this code generator
-"""
+language = writer.language  # Language generated by this code generator
