@@ -78,6 +78,11 @@ class CppBoxSizerBuilder(BaseCPPSizerBuilder):
     tmpl = '%(assignment)s = new %(klass)s(%(orient)s);\n'
 
 
+class CppWrapSizerBuilder(CppBoxSizerBuilder):
+    import_modules = ['<wx/wrapsizer.h>']
+    klass = 'wxWrapSizer'
+
+
 class CppStaticBoxSizerBuilder(BaseCPPSizerBuilder):
     klass = 'wxStaticBoxSizer'
     tmpl = '%(assignment)s = new %(klass)s(new wxStaticBox(%(parent_widget)s, wxID_ANY, %(label)s), %(orient)s);\n'
@@ -113,6 +118,7 @@ class CppSizerSlotGenerator(wcodegen.CppWidgetCodeWriter):
 def initialize():
     cn = common.class_names
     cn['EditBoxSizer'] = 'wxBoxSizer'
+    cn['EditWrapSizer'] = 'wxWrapSizer'
     cn['EditStaticBoxSizer'] = 'wxStaticBoxSizer'
     cn['EditGridSizer'] = 'wxGridSizer'
     cn['EditFlexGridSizer'] = 'wxFlexGridSizer'
@@ -121,6 +127,7 @@ def initialize():
     if cppgen:
         awh = cppgen.add_widget_handler
         awh('wxBoxSizer', CppBoxSizerBuilder())
+        awh('wxWrapSizer', CppWrapSizerBuilder())
         awh('wxStaticBoxSizer', CppStaticBoxSizerBuilder())
         awh('wxGridSizer', CppGridSizerBuilder())
         awh('wxFlexGridSizer', CppFlexGridSizerBuilder())
