@@ -120,10 +120,10 @@ class EditBase(EventsMixin, np.PropertyOwner):
         self.name  = name_p  = np.NameProperty(name)
         self.classname = klass
         self.klass = klass_p = np.ClassProperty(klass, name="class") # Name of the object's class: read/write or read only
-        if not custom_class: klass_p.readonly = True
+        if not custom_class: klass_p.readonly = True  # only used for StatusBar, ToolBar and also non-standalone MenuBar
 
         # Name of object's wxWidget class; base and klass are mostly the same, except e.g. wxDialog:
-        self.base = np.TextProperty(klass, "base")
+        self.base = klass  # not editable; e.g. wxFrame or wxComboBox; used to find the code generator
         # If true, the user can change the value of the 'class' property:
         self.custom_class = custom_class
 
@@ -170,15 +170,16 @@ class EditBase(EventsMixin, np.PropertyOwner):
         """Destructor. deallocates the popup menu, the notebook and all the properties.
         Why we need explicit deallocation? Well, basically because otherwise we get a lot of memory leaks... :)"""
         # XXX tell property editor
-
-        # ...finally, destroy our widget (if needed)
-        if self.widget and not self._dont_destroy:
-            if getattr(self, "sizer", None) and not self.sizer.is_virtual():
-                self.sizer.widget.Detach(self.widget)  # remove from sizer without destroying
-            compat.DestroyLater(self.widget)
-            self.widget = None
+        self.destroy_widget()
         if misc.focused_widget is self:
             misc.focused_widget = None
+    
+    def destroy_widget(self):
+        if not self.widget or self._dont_destroy: return
+        if getattr(self, "sizer", None) and not self.sizer.is_virtual():
+            self.sizer.widget.Detach(self.widget)  # remove from sizer without destroying
+        compat.DestroyLater(self.widget)
+        self.widget = None
 
     # context menu #####################################################################################################
     def popup_menu(self, event, pos=None):
@@ -205,36 +206,8 @@ class EditBase(EventsMixin, np.PropertyOwner):
         misc.bind_menu_item_after(widget, i, clipboard.cut, self)
         menu.AppendSeparator()
 
-        # rows/cols if inside a grid sizer
-        if "rows" in self.sizer.PROPERTIES:
-            row, col = self.sizer._get_row_col(self.pos)
-            i = misc.append_menu_item(menu, -1, _('Insert Row before') )
-            misc.bind_menu_item_after(widget, i, self.sizer.insert_row, self.pos)
-            i = misc.append_menu_item(menu, -1, _('Insert Column before') )
-            misc.bind_menu_item_after(widget, i, self.sizer.insert_col, self.pos)
-            if row==self.sizer.rows-1:
-                # last row
-                i = misc.append_menu_item(menu, -1, _('Add Row') )
-                misc.bind_menu_item_after(widget, i, self.sizer.insert_row, -1)
-            if col==self.sizer.cols-1:
-                # last col
-                i = misc.append_menu_item(menu, -1, _('Add Column') )
-                misc.bind_menu_item_after(widget, i, self.sizer.insert_col, -1)
-            menu.AppendSeparator()
-
-        if not self.sizer.is_virtual() and self.sizer._can_add_insert_slots():
-            # slots
-            i = misc.append_menu_item(menu, -1, _('Insert Slot before\tCtrl+I') )
-            misc.bind_menu_item_after(widget, i, self.sizer.insert_slot, self.pos)
-            i = misc.append_menu_item(menu, -1, _('Insert Slots before...\tCtrl+Shift+I') )
-            misc.bind_menu_item_after(widget, i, self.sizer.insert_slot, self.pos, True)
-
-            if self.pos==len(self.sizer.children)-1: # last slot -> allow to add
-                i = misc.append_menu_item(menu, -1, _('Add Slot\tCtrl+A') )
-                misc.bind_menu_item_after(widget, i, self.sizer.add_slot)
-                i = misc.append_menu_item(menu, -1, _('Add Slots...\tCtrl+Shift+A') )
-                misc.bind_menu_item_after(widget, i, self.sizer.add_slot, True)
-            menu.AppendSeparator()
+        if self.sizer and hasattr(self.sizer, "_add_popup_menu_items"):
+            self.sizer._add_popup_menu_items(menu, self, widget)
 
         # preview (create or close?)
         p = misc.get_toplevel_widget(self)
@@ -317,31 +290,6 @@ class EditBase(EventsMixin, np.PropertyOwner):
         The default implementation does nothing."""
         pass
 
-    def create_extracode_property(self):
-        try:
-            self.properties['extracode']._show(self.notebook)
-            self.properties['extraproperties']._show(self.notebook)
-        except KeyError:
-            pass
-
-    def set_property_blocking(self, key, item):
-        if key in self.property_blocking:
-            self.property_blocking[key].append(item)
-        else:
-            self.property_blocking[key] = [item]
-
-    def get_property_blocking(self, key):
-        if key in self.property_blocking:
-            return self.property_blocking[key]
-        return None
-
-    def remove_property_blocking(self, key, item):
-        if key in self.property_blocking:
-            for i in range(self.property_blocking[key].count(item)):
-                self.property_blocking[key].remove(item)
-            if not len(self.property_blocking[key]):
-                del self.property_blocking[key]
-
     @contextlib.contextmanager
     def frozen(self):
         if self.widget:
@@ -362,15 +310,22 @@ class WindowBase(EditBase):
     _PROPERTIES = ["id", "size", "background", "foreground", "font", "tooltip", "disabled", "focused", "hidden"]
     PROPERTIES = EditBase.PROPERTIES + _PROPERTIES
 
-    _PROPERTY_HELP = { "id":"""The "Id" property could be
-    1) a constant numeric value
-    2) a predefined identifier e.g. wxID_ANY
-    3) a predefined variable like a class member e.g. self.myButtonID
-    4) a variable assignment e.g. self.myButtonID=?
+    _PROPERTY_HELP = {
+        "id":   "The 'Id' property could be\n"
+                "    1) a constant numeric value\n"
+                "    2) a predefined identifier e.g. wxID_ANY\n"
+                "    3) a predefined variable like a class member e.g. self.myButtonID\n"
+                "    4) a variable assignment e.g. self.myButtonID=?\n\n"
+                "The pattern of a variable assignment is always 'variable=value'.\n\n"
+                "The value could be again a numeric value, a predefined identifier,\n"
+                "another predefined variable or '?' a shortcut for 'wxNewId()'.",
 
-    The pattern of a variable assignment is always "variable=value". The \
-    value could be again a numeric value, a predefined identifier, \
-    another predefined variable or "?" a shortcut for "wxNewId()"."""}
+        "size": "Specify the size of the widget.\n\n"
+                "E.g. '200,100' for a width of 200 pixels and a height of 100.\n\n"
+                "If you want to specify either width or height, you can use -1\n"
+                "for the other direction to keep the default for that.\n"
+                "E.g. '200,-1' or '-1,200'.\n"
+                "(The default value is '-1,-1' .)"}
 
     _PROPERTY_LABELS = {"attribute":'Store as attribute'}  # used in many derived widget editors
 
@@ -381,6 +336,8 @@ class WindowBase(EditBase):
         self.window_id = np.TextPropertyD( "wxID_ANY", name="id", default_value=None )
         self.size      = np.SizePropertyD( "-1, -1", default_value="-1, -1" )
 
+        self.sel_marker = None  # selection markers (a SelectionMarker instance)
+
         # background, foreground, font properties
         # their actual values will be stored/modified after widget creation in 'finish_widget_creation'
         # before that, the actual values will be stored in this dict from the actual values of the widget:
@@ -391,8 +348,11 @@ class WindowBase(EditBase):
         # font
         if "font" in self.PROPERTIES:
             self._font_changed = False # this is True if the user has selected a custom font
-            font = self._build_from_font( compat.wx_SystemSettings_GetFont(wx.SYS_DEFAULT_GUI_FONT) )
-            font[1] = 'default'
+            if config.use_gui:
+                font = self._build_from_font( compat.wx_SystemSettings_GetFont(wx.SYS_DEFAULT_GUI_FONT) )
+                font[1] = 'default'
+            else:
+                font = (9, 'default', 'normal', 'normal', 0, 'Segoe UI')
             self.font = np.FontPropertyD(tuple(font))
 
         # tooltip, focused, hiden
@@ -436,7 +396,11 @@ class WindowBase(EditBase):
         self.widget.Bind(wx.EVT_SIZE, self.on_size)
         # after setting various Properties, we must Refresh widget in order to see changes
         self.widget.Refresh()
-        self.widget.Bind(wx.EVT_KEY_DOWN, misc.on_key_down_event)
+        #self.widget.Bind(wx.EVT_KEY_DOWN, misc.on_key_down_event)
+        self.widget.Bind(wx.EVT_CHAR_HOOK, self.on_char_hook)
+    
+    def on_char_hook(self, event):
+        misc.handle_key_event(event, "design")
 
     def _reparent_widget(self, widget):
         "call Reparent(self.widget) for all direct children, including those in sizers"
@@ -616,8 +580,6 @@ class ManagedBase(WindowBase):
         # inside the sizer (proportion, borders, alignment...)
         self._has_layout = not sizer.is_virtual()
 
-        self.sel_marker = None  # selection markers (a SelectionMarker instance)
-
         # attributes to keep the values of the sizer properties
         self.pos        = np.LayoutPosProperty(pos)            # position within the sizer, 1-based
         self.span       = np.LayoutSpanProperty((1,1))         # cell spanning for GridBagSizer
@@ -695,16 +657,17 @@ class ManagedBase(WindowBase):
         size_p = self.properties["size"]
         if size_p.is_active() and size_p.get() != "-1, -1": return # fixed size
         # find best size, apply; display if size property is not active
+        self.widget.SetMinSize( (-1,-1) )  # otherwise the size would often not be reduced, e.g. for buttons
         best_size = self.widget.GetBestSize()
         self.sizer.set_item_best_size(self, best_size)
         if not size_p.is_active():
             size_p.set( best_size )
 
-    def delete(self):
+    def destroy_widget(self):
         if self.sel_marker:
             self.sel_marker.Destroy()  # destroy the selection markers
             self.sel_marker = None
-        WindowBase.delete(self)
+        WindowBase.destroy_widget(self)
 
     def _remove(self):
         "don't set focus"
@@ -737,7 +700,7 @@ class PreviewMixin(object):
         return self.preview_widget is not None
 
     def on_preview(self, refresh=False):
-        if self.preview_widget and config.debugging and wx.KeyboardState().ShiftDown():
+        if self.preview_widget and compat.IS_PHOENIX and config.debugging and wx.KeyboardState().ShiftDown():
             # print structure for debugging
             import utilities
             utilities.StructurePrinter(self.preview_widget)
@@ -804,8 +767,6 @@ class TopLevelBase(WindowBase, PreviewMixin):
         if wx.Platform == '__WXMSW__':
             # MSW isn't smart enough to avoid overlapping windows, so at least move it away from the 3 wxGlade frames
             self.widget.Center()
-        # ALB 2004-10-15
-        #self.widget.SetAcceleratorTable(common.palette.accel_table)
 
     def create(self):
         WindowBase.create(self)
@@ -841,7 +802,7 @@ class TopLevelBase(WindowBase, PreviewMixin):
 
         # paste
         i = misc.append_menu_item(menu, -1, _('Paste Sizer\tCtrl+V'), wx.ART_PASTE)
-        misc.bind_menu_item_after(widget, i, self.clipboard_paste)
+        misc.bind_menu_item_after(widget, i, clipboard.paste, self)
         if self.sizer is not None or not clipboard.check("sizer"): i.Enable(False)
 
         # preview
@@ -853,7 +814,7 @@ class TopLevelBase(WindowBase, PreviewMixin):
 
     def on_design_button(self):
         "Button 'Show Design Window' was pressed"
-        if self.widget and config.debugging and wx.KeyboardState().ShiftDown():
+        if self.widget and config.debugging and wx.GetKeyState(wx.WXK_SHIFT):
             import utilities
             utilities.StructurePrinter(self.widget)
             return
@@ -878,7 +839,7 @@ class TopLevelBase(WindowBase, PreviewMixin):
             return (True,None)
         import edit_sizers
         if not isinstance(widget, edit_sizers.Sizer):
-            return 'Only sizers can be pasted here'
+            return (False, 'Only sizers can be pasted here')
         return (True,None)
 
     def clipboard_paste(self, clipboard_data):
@@ -944,13 +905,12 @@ class TopLevelBase(WindowBase, PreviewMixin):
 
     def delete(self, *args):
         if self.preview_widget is not None:
-            if misc.preview_event_filter:
-                self.preview_widget.RemoveFilter(misc.preview_event_filter)
+            self.preview_widget.Unbind(wx.EVT_CHAR_HOOK)
             compat.DestroyLater(self.preview_widget)
             self.preview_widget = None
         widget = self.widget
         WindowBase.delete(self)
-        if widget is not None: misc.design_windows.remove(widget)
+        if widget is not None and widget in misc.design_windows: misc.design_windows.remove(widget)
 
     def _find_widget_by_pos(self, w, x,y, level=1):
         "helper for find_widget_by_pos; w is the parent window/widget"
@@ -1033,7 +993,7 @@ class EditStylesMixin(np.PropertyOwner):
         # set code generator only once per class
         if not self.codegen:
             self.codegen = common.code_writers['python'].copy()
-            self.codegen.for_version = wx.VERSION[0:2]
+            self.codegen.for_version = compat.version
             EditStylesMixin.codegen = self.codegen
 
         try:
@@ -1061,7 +1021,11 @@ class EditStylesMixin(np.PropertyOwner):
         new_style = self.style
         if old_style == new_style: return
 
-        if not self.recreate_on_style_change:
+        recreate = self.recreate_on_style_change
+        if self.base == "wxButton" and (old_style & wx.BU_EXACTFIT != new_style & wx.BU_EXACTFIT):
+            recreate = True  # workaround
+
+        if not recreate:
             # update style without re-creating the widget
             self.widget.SetWindowStyleFlag(new_style)
             self.widget.Refresh()
@@ -1069,23 +1033,24 @@ class EditStylesMixin(np.PropertyOwner):
 
         # some widgets can't be updated, e.g. Gauge can't be switched between horizontal and vertical after creation
         # this is for ManagedBase derived classes only
-        focused = misc.focused_widget is self
-        if self.sel_marker:
-            self.sel_marker.Destroy()
-            self.sel_marker = None
-        old_widget = self.widget
-        old_widget.Hide()
-        si = self.sizer.widget.GetItem(old_widget)
-        self.create_widget()
-        compat.SizerItem_SetWindow(si, self.widget)
-        compat.DestroyLater(old_widget)
-        self.sizer.item_properties_modified(self)
+        with self.frozen():
+            focused = misc.focused_widget is self
+            if self.sel_marker:
+                self.sel_marker.Destroy()
+                self.sel_marker = None
+            old_widget = self.widget
+            old_widget.Hide()
+            si = self.sizer.widget.GetItem(old_widget)
+            self.create_widget()
+            compat.SizerItem_SetWindow(si, self.widget)
+            compat.DestroyLater(old_widget)
+            self.sizer.item_properties_modified(self)  # will call toplevel Refresh as well
 
-        self.finish_widget_creation(re_add=False)
-        self.sizer.layout()
-        if focused:
-            misc.focused_widget = self
-            if self.sel_marker: self.sel_marker.Show(True)
+            self.finish_widget_creation(re_add=False)
+            self.sizer.layout()
+            if focused:
+                misc.focused_widget = self
+                if self.sel_marker: self.sel_marker.Show(True)
 
 
     @decorators.memoize
