@@ -21,6 +21,7 @@ class Node(object):
         self.widget = widget      # e.g. EditPanel or EditBoxSizer
         self.children = children  # list of Node or SlotNode instances
         self.parent = None        # parent node; will be set in Tree.add/insert
+        self.item = None
     @property
     def is_toplevel(self):
         if self.parent.parent is None:
@@ -46,7 +47,7 @@ class Node(object):
             pass
 
     def has_ancestor(self, node):
-        # returns True if node is parent or parents parent ...
+        "Returns True if node is parent or parents parent ..."
         parent = self.parent
         if parent is None: return False
         while True:
@@ -105,7 +106,7 @@ class Node(object):
         output.append(u'%s</object>\n' % outer_tabs)
 
     def get_image(self, image=None):
-        # get an image for this node
+        "Get an image for this node"
         if image is not None:
             return image
 
@@ -135,6 +136,11 @@ class Node(object):
                 name = "EditHorizontalSizer"
             else:
                 name = "EditSpacer"
+        elif name=="EditWrapSizer":
+            if widget.orient & wx.VERTICAL:
+                name = "EditVerticalWrapSizer"
+            else:
+                name = "EditHorizontalWrapSizer"
 
         elif name == "EditSplitterWindow":
             if widget.orientation=="wxSPLIT_HORIZONTAL":
@@ -217,17 +223,29 @@ class Tree(object):
         self.names.setdefault(self._find_toplevel(child), {})[child.widget.name] = 1
         if parent is self.root and getattr(child.widget.__class__, '_is_toplevel_window', False):
             self.app.add_top_window(child.widget.name)
+            
+        if not isinstance(child.widget, edit_sizers.SizerSlot):
+            self.app.check_codegen(child.widget)
 
     def insert(self, child, parent, index):
         # if at index there is a SlotNode, replace this, otherwise insert
         if parent is None: parent = self.root
-        if parent.children is None: parent.children = []
+        if parent.children is None or index>=len(parent.children):
+            self.add(child, parent)
+            return
+        if isinstance( parent.children[index], SlotNode ):
+            if not isinstance(child, SlotNode):
+                self.remove( parent.children[index] )
+
         parent.children.insert(index, child)
         child.parent = parent
         self.current = child
         self.names.setdefault(self._find_toplevel(child), {})[child.widget.name] = 1
         if parent is self.root:
             self.app.add_top_window(child.widget.name)
+            
+        if not isinstance(child.widget, edit_sizers.SizerSlot):
+            self.app.check_codegen(child.widget)
 
     def clear_name_rec(self, n):
         if n.parent is self.root:
@@ -261,7 +279,7 @@ class Tree(object):
         output.append( u'<?xml version="1.0"?>\n'
                        u'<!-- generated by wxGlade %s on %s -->\n\n' % (config.version, timestring) )
 
-        attrs = ["name","class","language","top_window","encoding","use_gettext", "overwrite",
+        attrs = ["name","class","language","top_window","encoding","use_gettext", "overwrite", "mark_blocks",
                  "for_version","is_template","indent_amount"]
         props = [self.app.properties[attr] for attr in attrs]
         attrs = dict( (attr,prop.get_string_value()) for attr,prop in zip(attrs,props) if not prop.deactivated )
@@ -359,8 +377,35 @@ class Tree(object):
         if start_node is None: start_node = self.root
         self._prn(start_node, 0)
 
+    def clear(self):
+        # delete all children; call common.app_tree.app.new() or .init() afterwards
+        self.skip_select = True
+        if self.root.children:
+            while self.root.children:
+                c = self.root.children[-1]
+                if c.widget: c.widget.remove()
+            self.root.children = None
+        self.skip_select = False
 
+    def set_title(self, value):
+        if value is None: value = ""
+        self.title = value
 
+    def get_title(self):
+        if not self.title: self.title = ' '
+        return self.title
+
+    def refresh_name(self, node, previous_name=None):
+        names = self.names.setdefault(self._find_toplevel(node), {})
+        if previous_name is not None and previous_name in names:
+            del names[previous_name]
+        names[node.widget.name] = 1
+
+    def refresh(self, node, refresh_label=True, refresh_image=True):
+        # nothing to, do when there is no GUI
+        pass
+        
+    
 class WidgetTree(wx.TreeCtrl, Tree):
     "Tree with the ability to display the hierarchy of widgets"
     images = {} # Dictionary of icons of the widgets displayed
@@ -406,7 +451,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
         self.Bind(wx.EVT_TREE_END_LABEL_EDIT, self.end_edit_label)
         #self.Bind(wx.EVT_KEY_DOWN, misc.on_key_down_event)
         self.Bind(wx.EVT_KEY_DOWN, self.on_key_down_event)
-        self.Bind(wx.EVT_CHAR_HOOK, self.on_char)
+        #self.Bind(wx.EVT_CHAR_HOOK, self.on_char)  # on wx 2.8 the event will not be delivered to the child
 
     def _label_editable(self, widget=None):
         if widget is None: widget = self.cur_widget
@@ -420,12 +465,16 @@ class WidgetTree(wx.TreeCtrl, Tree):
         return True
 
     def on_char(self, event):
-        "start label editing on F2"
+        "called from main: start label editing on F2; skip events while editing"
         if event.GetKeyCode()==wx.WXK_F2 and self.cur_widget and self._label_editable():
             # start label editing
             self.EditLabel( self.cur_widget.node.item )
-            return
-        event.Skip()
+            return True
+        if isinstance(self.FindFocus(), wx.TextCtrl):
+            # currently editing
+            event.Skip()
+            return True
+        return False
 
     def on_key_down_event(self, event):
         #if event.GetKeyCode()==13:
@@ -441,7 +490,8 @@ class WidgetTree(wx.TreeCtrl, Tree):
                     pos = (rect.right, (rect.top+rect.bottom)//2)
                     self.cur_widget.popup_menu(event, pos=pos)
                     return
-        misc.on_key_down_event(event)
+        #misc.on_key_down_event(event)
+        event.Skip()
 
     def begin_drag(self, evt):
         # start drag & drop
@@ -469,7 +519,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
         elif new_value.endswith('"') and ': "' in new_value:
             new_name, new_label = new_value.split(': "', 1)
             new_label = new_label[:-1]
-        elif new_value and new_value[0] in ["'",'"'] or new_value[-1] in ["'",'"']:
+        elif new_value and (new_value[0] in ["'",'"'] or new_value[-1] in ["'",'"']):
             # just a label; for this we accept errors, we just want at least one quotation mark
             if new_value[0]==new_value[-1] and len(new_value)>2:
                 new_label = new_value[1:-1]
@@ -618,13 +668,13 @@ class WidgetTree(wx.TreeCtrl, Tree):
             label = node.widget.label
             label = label.replace("\n","\\n").replace("\t","\\t")
             if '"' in label:
-                if len(label)>24:
-                    s += ": '%s..."%(label[:18])
+                if len(label)>36:
+                    s += ": '%s..."%(label[:30])
                 else:
                     s += ": '%s'"%label
             else:
                 if len(label)>24:
-                    s += ': "%s...'%(label[:18])
+                    s += ': "%s...'%(label[:30])
                 else:
                     s += ': "%s"'%label
         elif getattr(node.widget, "has_title", None):
@@ -642,12 +692,18 @@ class WidgetTree(wx.TreeCtrl, Tree):
         def _SetItemData(self, item, data):
             self.SetPyData(item, data)
         def _GetItemData(self, item):
+            if not bool(item): return None
             return self.GetPyData(item)
     else:
         def _SetItemData(self, item, data):
             self.SetItemData(item, data)
         def _GetItemData(self, item):
-            return self.GetItemData(item)
+            if not bool(item): return None
+            try:
+                return self.GetItemData(item)
+            except RuntimeError:
+                # on GTK the above bool may return True even if the item is being deleted
+                return None
 
     def add(self, child, parent=None, select=True):
         "appends child to the list of parent's children"
@@ -663,8 +719,6 @@ class WidgetTree(wx.TreeCtrl, Tree):
             if select:
                 self.select_item(child)
 
-        if not isinstance(child.widget, edit_sizers.SizerSlot):
-            self.app.check_codegen(child.widget)
 
     def insert(self, child, parent, index, select=True):
         "inserts child to the list of parent's children at index; a SlotNode at index is overwritten"
@@ -680,10 +734,6 @@ class WidgetTree(wx.TreeCtrl, Tree):
             self.add(child, parent, select=select)
             return
 
-        if isinstance( parent.children[index], SlotNode ):
-            if not isinstance(child, SlotNode):
-                self.remove( parent.children[index] )
-
         Tree.insert(self, child, parent, index)
         image = child.get_image()
         child.item = compat.wx_Tree_InsertItemBefore(self, parent.item, index, self._build_label(child), image)
@@ -692,8 +742,6 @@ class WidgetTree(wx.TreeCtrl, Tree):
             self.Expand(parent.item)
             if select:
                 self.select_item(child)
-        if not isinstance(child.widget, edit_sizers.SizerSlot):
-            self.app.check_codegen(child.widget)
 
     def remove(self, node=None, delete=True):
         self.app.saved = False  # update the status of the app
@@ -704,21 +752,8 @@ class WidgetTree(wx.TreeCtrl, Tree):
         else:
             wx.TreeCtrl.Destroy(self)
 
-    def clear(self):
-        # delete all children; call common.app_tree.app.new() or .init() afterwards
-        self.skip_select = True
-        if self.root.children:
-            while self.root.children:
-                c = self.root.children[-1]
-                if c.widget: c.widget.remove()
-            self.root.children = None
-        self.skip_select = False
-
     def refresh_name(self, node, previous_name=None):
-        names = self.names.setdefault(self._find_toplevel(node), {})
-        if previous_name is not None and previous_name in names:
-            del names[previous_name]
-        names[node.widget.name] = 1
+        Tree.refresh_name(self, node, previous_name)
         self.SetItemText(node.item, self._build_label(node))
 
     def refresh(self, node, refresh_label=True, refresh_image=True):
@@ -733,9 +768,25 @@ class WidgetTree(wx.TreeCtrl, Tree):
         self.skip_select = True
         self.SelectItem(node.item)
         self.skip_select = False
-        self.cur_widget = node.widget
+        self._set_cur_widget(node.widget)
         misc.set_focused_widget(self.cur_widget)
 
+    def _set_cur_widget(self, widget):
+        # set self.cur_widget; adjust label colors and bold if required (on Windows)
+        if self.cur_widget and wx.Platform == "__WXMSW__":
+            item = self.cur_widget.node.item
+            self.SetItemTextColour(item, wx.NullColour)
+            self.SetItemBold( item, False )
+        self.cur_widget = widget
+        item = widget.node.item
+        self.EnsureVisible(item)
+        # ensure that the icon is visible
+        text_rect = self.GetBoundingRect(item, True)
+        if text_rect.x<22:
+            self.SetScrollPos(wx.HORIZONTAL, self.GetScrollPos(wx.HORIZONTAL) - 22 + text_rect.x)
+        if wx.Platform == "__WXMSW__":
+            self.SetItemBold(item, True)
+            self.SetItemTextColour(item, wx.BLUE)
     def set_current_widget(self, widget):
         # interface from common.set_focused_widget
         if widget is None or widget is self.cur_widget: return
@@ -746,13 +797,13 @@ class WidgetTree(wx.TreeCtrl, Tree):
         self.skip_select = True
         self.SelectItem(node.item)
         self.skip_select = False
-        self.cur_widget = widget
+        self._set_cur_widget(widget)
 
     def on_change_selection(self, event):
         if self.skip_select: return  # triggered by self.SelectItem in self.set_current_widget
         item = event.GetItem()
         widget = self._GetItemData(item).widget
-        self.cur_widget = widget
+        self._set_cur_widget(widget)
         misc.set_focused_widget(widget)
         if not self.IsExpanded(item):
             self.Expand(item)
@@ -839,17 +890,12 @@ class WidgetTree(wx.TreeCtrl, Tree):
         else: self.Collapse(node.item)
 
     def set_title(self, value):
-        if value is None: value = ""
-        self.title = value
+        Tree.set_title(self, value)
         try:
-            self.GetParent().SetTitle(_('wxGlade: Tree %s') % value)
+            self.GetTopLevelParent().SetTitle(_('wxGlade: %s') % value)
         except:
             pass
 
-    def get_title(self):
-        if not self.title: self.title = ' '
-        return self.title
-
     def create_widgets(self, node):
         "Shows the widget of the given node and all its children"
         node.widget.create()
@@ -866,9 +912,10 @@ class WidgetTree(wx.TreeCtrl, Tree):
         if not node.widget.widget:
             node.widget.create_widget()
             node.widget.finish_widget_creation()
-            node.widget.drop_target = clipboard.DropTarget(node.widget)
-            node.widget.widget.SetDropTarget(node.widget.drop_target)
-        
+            if not node.widget.__class__.__name__ in ('EditMenuBar', 'EditToolBar'):
+                node.widget.drop_target = clipboard.DropTarget(node.widget)
+                node.widget.widget.SetDropTarget(node.widget.drop_target)
+
         with node.widget.frozen():
             if node.children:
                 for c in node.children:
@@ -879,9 +926,8 @@ class WidgetTree(wx.TreeCtrl, Tree):
                 node.widget.widget.Show()
             else:
                 node.widget.widget.GetParent().Show()
-    
-            misc.set_focused_widget(node.widget)
-    
+
+            #misc.set_focused_widget(node.widget)
             node.widget.widget.Raise()
             # set the best size for the widget (if no one is given)
             props = node.widget.properties
@@ -933,8 +979,8 @@ class WidgetTree(wx.TreeCtrl, Tree):
             toplevel_widget.GetTopLevelParent().Hide()
 
             # added by rlawson to collapse only the toplevel node, not collapse back to root node
-            self.select_item(node)
-            misc.set_focused_widget(node.widget)
+            #self.select_item(node)
+            #misc.set_focused_widget(node.widget)
             if event: event.Skip()
         if "design" in node.widget.properties: node.widget.design.update_label()
 
@@ -1063,7 +1109,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
                 itemok = item
                 if parent is None:
                     parent = self._GetItemData(itemok)
-                self.cur_widget = widget
+                self._set_cur_widget(widget)
                 item, cookie = self._get_first_child(item)
                 index += 1
             else:
@@ -1072,6 +1118,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
             node = self._GetItemData(itemok)
             if parent is not None:
                 self._show_widget_toplevel(parent)
+                common.app_tree.expand(parent)
                 if pos is not None:
                     misc.get_toplevel_parent(parent.widget).SetPosition(pos)
             self.select_item(node)
