@@ -8,19 +8,21 @@ wxButton objects
 """
 
 import wx
-import config, common
+import config, common, compat
 from edit_windows import ManagedBase, EditStylesMixin
 from tree import Node
 import new_properties as np
 from .button_stockitems import *
+from gui_mixins import BitmapMixin
 
 
-class EditButton(ManagedBase, EditStylesMixin):
+class EditButton(ManagedBase, EditStylesMixin, BitmapMixin):
     "Class to handle wxButton objects"
 
     STOCKITEMS = sorted( ButtonStockItems.stock_ids.keys())
-
-    _PROPERTIES = ["Widget", "label", "stockitem", "default", "style"]
+    _PROPERTIES = ["Widget", "label", "stockitem",
+                   "bitmap", "disabled_bitmap", "pressed_bitmap", "current_bitmap", "focus_bitmap",
+                   "default", "style"]
     PROPERTIES = ManagedBase.PROPERTIES + _PROPERTIES + ManagedBase.EXTRA_PROPERTIES
 
     _PROPERTY_HELP = {"default":"This sets the button to be the default item for the panel or dialog box.",
@@ -30,12 +32,19 @@ class EditButton(ManagedBase, EditStylesMixin):
         # Initialise parent classes
         ManagedBase.__init__(self, name, 'wxButton', parent, id, sizer, pos)
         EditStylesMixin.__init__(self)
+        BitmapMixin.__init__(self)
 
         # initialise instance properties
         self.label     = np.TextProperty(label, default_value="", multiline="grow")
         self.default   = np.CheckBoxProperty(False, default_value=False)
         self.stockitem = np.ComboBoxPropertyD(self.STOCKITEMS[0], choices=self.STOCKITEMS)
 
+        self.bitmap          = np.BitmapPropertyD(min_version=(3,0))
+        self.disabled_bitmap = np.BitmapPropertyD(min_version=(3,0))
+        self.pressed_bitmap  = np.BitmapPropertyD(min_version=(3,0))
+        self.current_bitmap  = np.BitmapPropertyD(min_version=(3,0))
+        self.focus_bitmap    = np.BitmapPropertyD(min_version=(3,0))
+
     def create_widget(self):
         stockitem_p = self.properties["stockitem"]
         if stockitem_p.is_active():
@@ -43,12 +52,13 @@ class EditButton(ManagedBase, EditStylesMixin):
         else:
             label = self.label
         self.widget = wx.Button(self.parent.widget, self.id, label, style=self.style)
+        if compat.IS_PHOENIX:
+            self._set_preview_bitmaps()
 
     def properties_changed(self, modified=None):
         "update label (and size if label/stockitem have changed)"
 
         label_modified = not modified or "label" in modified
-        resize = False
 
         if not modified or "stockitem" in modified:
             # if stockitem is set, label needs to be deactivated and window id is wxID_...
@@ -61,7 +71,6 @@ class EditButton(ManagedBase, EditStylesMixin):
                 l = ButtonStockItems.stock_ids[self.stockitem]
                 if self.widget:
                     self.widget.SetLabel(l)
-                    resize = True
             else:
                 self.properties["label"].set_blocked(False)
                 #self.properties["id"].default_value = "wxID_ANY"
@@ -70,13 +79,12 @@ class EditButton(ManagedBase, EditStylesMixin):
         if label_modified and self.properties["label"].is_active():
             if self.widget:
                 self.widget.SetLabel(self.label)
-                resize = True
 
         if label_modified or "name" in modified:
             common.app_tree.refresh(self.node, refresh_label=True)
 
-        if resize and self.widget: self._set_widget_best_size()
-
+        BitmapMixin._properties_changed(self, modified)
+        self._set_widget_best_size()
         EditStylesMixin.properties_changed(self, modified)
         ManagedBase.properties_changed(self, modified)
 
