@@ -10,18 +10,14 @@ Lisp generator functions for wxCheckListBox objects
 
 import common
 import wcodegen
-from ChoicesCodeHandler import *
 
 
 class LispCheckListBoxGenerator(wcodegen.LispWidgetCodeWriter):
     tmpl = '(setf %(name)s (%(klass)s_Create %(parent)s %(id)s -1 -1 -1 ' \
            '-1 %(choices_len)s (vector %(choices)s) %(style)s))\n'
 
-# end of class LispCheckListBoxGenerator
-
 
 def initialize():
     klass = 'wxCheckListBox'
     common.class_names['EditCheckListBox'] = klass
-    common.register('lisp', klass, LispCheckListBoxGenerator(klass),
-                    'choices', ChoicesCodeHandler)
+    common.register('lisp', klass, LispCheckListBoxGenerator(klass) )
