@@ -18,7 +18,7 @@ class CheckBoxMixin(object):
         @param obj: Instance of L{xml_parse.CodeObject}
         @type obj: xml_parse.CodeObject
         """
-        checked = int(obj.properties.get('checked', 0))
+        checked = obj.checked
         if self.cn_f('wxCHK_3STATE') in self.tmpl_dict['style']:
             checked = self.config['number2state'][checked]
             checked = self.cn_f(checked)
