@@ -12,10 +12,20 @@ import wcodegen
 
 class PythonStaticTextGenerator(wcodegen.PythonWidgetCodeWriter):
     tmpl = '%(name)s = %(klass)s(%(parent)s, %(id)s, %(label)s%(style)s)\n'
+    def get_more_properties_code(self, obj):
+        ret = []
+        if obj.wrap>0:
+            ret.append( '%s.Wrap(%d)\n'%(self.tmpl_dict['name'], obj.wrap) )
+        return ret
 
 
 class CppStaticTextGenerator(wcodegen.CppWidgetCodeWriter):
     tmpl = '%(name)s = new %(klass)s(%(parent)s, %(id)s, %(label)s%(style)s);\n'
+    def get_more_properties_code(self, obj):
+        ret = []
+        if obj.wrap>0:
+            ret.append( '%s->Wrap(%d);\n'%(obj.name, obj.wrap) )
+        return ret
 
 
 def xrc_code_generator(obj):
