@@ -3,7 +3,7 @@ wxToolBar objects
 
 @copyright: 2002-2007 Alberto Griggio
 @copyright: 2014-2016 Carsten Grohmann
-@copyright: 2017-2018 Dietmar Schwertberger
+@copyright: 2017-2019 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
@@ -457,7 +457,9 @@ class ToolsProperty(np.Property):
         else:
             parent = None
         dialog = ToolsDialog( parent, self.owner, items=self.value )
-        if dialog.ShowModal() == wx.ID_OK:
+        with misc.disable_stay_on_top(common.adding_window or parent):
+            res = dialog.ShowModal()
+        if res == wx.ID_OK:
             self.on_value_edited(dialog.get_items())
         dialog.Destroy()
 
